from requests import Response, Session
from time import sleep

headers = {'Content-type': 'application/json', 'Accept': 'application/json'}

class Connections():
    @staticmethod
    def get_api(session: Session, url: str, params: dict=None) -> Response:
        while True:
            r = session.get(url, params=params, headers=headers)

            if not Connections.api_rate_limited(r.status_code) and Connections.authorized(r.status_code):
                return r

    @staticmethod
    def put_api(session: Session, url: str, params: dict=None) -> Response:
        while True:
            r = session.put(url, params=params, headers=headers)

            if not Connections.api_rate_limited(r.status_code) and Connections.authorized(r.status_code):
                return r

    @staticmethod
    def post_api(session: Session, url: str, params: dict=None, payload: dict=None) -> Response:
        while True:
            r = session.post(url, params=params, json=payload, headers=headers)

            if not Connections.api_rate_limited(r.status_code) and Connections.authorized(r.status_code):
                return r

    @staticmethod
    def delete_api(session: Session, url: str, params: dict=None) -> Response:
        while True:
            r = session.delete(url, params=params, headers=headers)

            if not Connections.api_rate_limited(r.status_code) and Connections.authorized(r.status_code):
                return r

    @staticmethod
    def api_rate_limited(status_code: int) -> bool:
        if status_code == 429:
            print('WARN: Hit api rate limit, sleeping for 1 minute then attempting to resume...')
            sleep(60)
            return True
        return False

    @staticmethod
    def authorized(status_code: int) -> bool:
        if status_code == 401:
            exit('FATAL: Received 401 response indicating misspelled/invalid credentials, '
                 'please check your credentials and try again. Closing...')
        elif status_code == 403:
            exit('FATAL: Received 403 response indicating missing permissions, '
                 'please ensure you have granted yourself site admin and try again. Closing...')
        return True

    @staticmethod
    def paged(next_page: str, page: int):
        if next_page is None:
            return None

        if page is None:
            page = 1
        return page + 1
